/*
  --> MikMod 3.0
   -> Release Date 09/20/97
   
  This example demonstrates how to create a simple module player.  If you
  are interested in a more advanced interface (such as a game) which would
  require the use of sound effects, see "MIKSOUND.C"

  This example file uses a 'polling' method of sound update.  This method
  is very portable, and is Windows friendly, but is not usually the best
  way to handle sound updating in many DOS applications.  See MIKINTER.C
  for the interrupt driven example of a simple module player.

  This version of the MikMod 3.0 library is still in the initial stages,
  meaning that it still has some sucky "features" (yes, bugs :), but it's
  pretty functional otherwise.  New releases should be made periodically.

  For more information concerning the use of MikMod 3.0, see the documen-
  tation in the /DOCS directory.

---------

  Keep a look out for updates and new betas!

  mailto:dracoirs@epix.net
  http://www.epix.net/~dracoirs/
 
*/

#include "types.h"
#include "mikmod.h"

#define ESCAPE     27

#define F1         59
#define F2         60
#define F3         61
#define F4         62
#define F5         63
#define F6         64
#define F7         65
#define F8         66
#define F9         67
#define F10        68


#define LIST_LOADERS  1
#define LIST_DRIVERS  2


// strings are defined in message.c
extern CHAR banner[], helptext01[], helptext02[], presskey[];
extern char modulefileptr[];
extern char modulefileendptr[];
extern char logo[];

// Globals that would likely be used outside the main module

int    cursong;
UNIMOD *mf;               // module handle / format information.  See
                          // "mikmod.h" for a description of its contents.


// Static Module Variables.
// ------------------------
// The following variables can be modified via command line options.

static BOOL  morehelp     = 0,
             m15_enabled  = 0,     // set = registers 15-inst module loader
             emu_enabled  = 1;     // set = display mod status line

static int   ldlist       = 0;     // List Loaders / Drivers?
static int   cfg_maxchn   = 64;    // max channels that can be allocated by a module

static BOOL  cfg_extspd  = 1,      // Extended Speed enable
             cfg_panning = 1,      // DMP panning enable (8xx effects)
             cfg_loop    = 1;      // auto song-looping enable


void errorhandler(void)

// When called, the following global variables will have been set:
//  _mm_errno     - INTEGER; the error that occured [listed in mmio.h]
//  _mm_critical  - BOOLEAN; set to TRUE if an error occurs within
//                  MikMod_Init() or when trying to start playing of a
//                  module - in which case MikMod automatically shuts
//                  down and impliments the NoSound driver.

{

    // printf("MikMod Error",_mm_errmsg[_mm_errno]);
    puts("MikMod Error :");
    puts(_mm_errmsg[_mm_errno]);

    if(_mm_critical)
      {   // exit(_mm_errno);     // Mod player not much use without sound, so
                             // just exit the program.
    }
}

void display_logo(void) {
  int itx, ity; 

  for (ity = 0; ity < 82; ity++) {
    for (itx = 0; itx < 320; itx++) {
      *((u8 *)(0xA0300000+itx*4+ity*320*4))   = *((u8 *)(logo+itx*3+ity*320*3+2));
      *((u8 *)(0xA0300000+itx*4+ity*320*4+1)) = *((u8 *)(logo+itx*3+ity*320*3+1));
      *((u8 *)(0xA0300000+itx*4+ity*320*4+2)) = *((u8 *)(logo+itx*3+ity*320*3));
      *((u8 *)(0xA0300000+itx*4+ity*320*4+3)) = 0; // *((u8 *)(LOGO+itx*4+ity*319*4  ));
    }
  }
}

void boot(void)
{
   BOOL  quit = 0,     // set true when time to exit modplayer (escape pressed)
         next;         // set true when time to play next song (space pressed)
   int loops = 0;
   unsigned int it;
   u32 value1, value2, value3, value4, result;
   u8  v1, v2, v3, v4;
   u32 beginline, endline, maxlines = 0;

   initscrbuf((unsigned char *)0x80300000);

   display_logo();

   setcpos(30, 82);
   puts("CZN Module Player v3 by Loom", 0x0);

   // Register the MikMod error handler.  If any errors occur within the
   // MikMod libraries (including the ngetopt and all _mm_ functions),
   // the given error handler will be called and _mm_errno set. [see
   // errorhandler(void) above]

   MikMod_RegisterErrorHandler(errorhandler);

   // Initialize soundcard parameters.
   // --------------------------------
   // you _have_ to do this before calling MikMod_Init().  Changing them
   // afterward will result in unpredictable behavior, so don't do it!  You
   // can change these values if you call MikMod_Exit(); and then
   // MikMod_Init() again.
   // Note that the values below are the default assignments as found in
   // MDRIVER.C, and are unneccessary here if you plan to use the defaults
   // only.

   md_mixfreq      = 44100;            // standard mixing freq
   md_dmabufsize   = 40000;            // DMA buffer size of 4 seconds (default = 50 millseconds)
   md_device       = 0;                // standard device: autodetect
   md_volume       = 16;               // driver volume (max 128)
   md_musicvolume  = 128;              // music volume (max 128)
   md_sndfxvolume  = 128;              // sound effects volume (max 128)
   md_pansep       = 128;              // panning separation (0 = mono, 128 = full stereo)
   md_reverb       = 0;                // Reverb (max 15)


   // Device Mixing Mode
   // ------------------
   // Available supported options (more in the next beta):
   //
   //   DMODE_STEREO       - Enables stereo output
   //   DMODE_16BITS       - Enables 16 bits output (see VIRTCH.C for enabling
   //                        16 bit software mixing)
   //   DMODE_REVERSE      - Turns on Reverse Stereo (Left<->Right)
   //   DMODE_SOFT_SNDFX   - Force software mixing of sound effects
   //   DMODE_SOFT_MUSIC   - Force software mixing of music
   
   md_mode = DMODE_16BITS | DMODE_STEREO | DMODE_SURROUND;

   //   puts(banner);     

   // ==========================================
   //    MikMod Loader / Driver Registration!
   // ==========================================

   MikMod_RegisterAllLoaders();

   MikMod_RegisterAllDrivers();
   MikMod_RegisterDriver(drv_n64);     // raw driver - dumps output to a raw file
   MikMod_RegisterDriver(drv_nos);

    // ===================================================================
    // Initialize MikMod (initializes soundcard and associated mixers, etc)
    // NOTE: Errors here are now handled by the error handler above.  How-
    //       ever, MikMod_Init() still returns the boolean value TRUE on
    //       error.

    puts("Setting up memory system");
    initmem();
    *((u32*)0xa4500010)=0x00000465;
    *((u32*)0xa450000c)=0x0000000f;


    puts("Initialising");
    MikMod_Init();

    //    if (md_mode & DMODE_16BITS) puts("16 BIT MODE"); else puts("8 BIT MODE");
    //    if (md_mode & DMODE_INTERP) puts("INTERPOLATION ACTIVE"); else puts("NO INTERPOLATION");
    //    if (md_mode & DMODE_STEREO) puts("STEREO PLAYBACK"); else puts("MONO PLAYBACK");
    //    printf("Using %s for %d bit %s %s sound at %u Hz\n\n",
    //       md_driver->Name,
    //       (md_mode&DMODE_16BITS) ? 16:8,
    //       (md_mode&DMODE_INTERP) ? "interpolated" : "normal",
    //       (md_mode&DMODE_STEREO) ? "stereo" : "mono",
    //       md_mixfreq);

    // printf("File    : %s\n",argv[cursong]);

        // To load a module, pass the module name, and the MAXIMUM number
        // of channels that is allowed to be allocated.  MikMod_LoadSong will
        // allocate the number of channels needed (dependant on module type,
        // as non-IT formats do not require more than pf->numchn channels).

        // Note that if an error occurs, it is handled by the error handler.
        // Any critical errors will exit the player, other errors can simply
        // skip the song.


	puts("Loading module..");
        if((mf=MikMod_LoadSong(modulefileptr,(modulefileendptr - modulefileptr + 1), 32)) == NULL) {
           puts("Error loading module");
           puts("Execution halted");
           while (1) {};
        }

        // Set up the module playback information.
        // These values can be changed at any time!
        // See MIKMOD.TXT or MIKMOD.H for a description of the various
        // elements in the UNIMOD structure.

        mf->extspd  = cfg_extspd;
        mf->panflag = cfg_panning;
        mf->loop    = cfg_loop;

        // printf("Songname: %s\n"
        //       "Modtype : %s\n"
        //       "Periods : %s,%s\n"
        //       "Channels: %d (song), %d (mixed)\n",
        //       mf->songname,
        //       mf->modtype,
        //       (mf->flags & UF_XMPERIODS) ? "XM type" : "mod type",
        //       (mf->flags & UF_LINEAR) ? "Linear" : "Log",
        //       mf->numchn,md_numchn);
	puts("Songname : ");
        puts(mf->songname);
        puts("Mod type : ");
        puts(mf->modtype);
        if (mf->flags & UF_XMPERIODS) 
          puts("XM type");
	else
          puts("Mod type");
        if (mf->flags & UF_LINEAR) 
          puts("Linear volumes");
        else
          puts("Logarithmic volumes");

	printf("Channels", mf->numchn);
	puts("Playback start...");
        Player_Start(mf);      // start playing the module

	while(Player_Active()) 
        {  

	    //setscancol(0);
	  //            beginline = getscanline();
            MikMod_Update();
	    //setscancol(0xFFFFFFFF);
	    //            endline = getscanline();
	    //            if (((endline-beginline) > maxlines) && (endline > beginline)) {
	    //  	      setcpos(30, 240 - 100);
	    //              maxlines = endline - beginline;
	    //              printf("TIME", maxlines);
	    //            }
            loops++;
	   
            for (it = 82*320*4; it < 320*240*4; it += 4) {

              value1 = *((u32 *)(0x80300000-320*4+it));
              value2 = *((u32 *)(0x80300000+320*4+it));
              value3 = *((u32 *)(0x80300000-4+it));
              value4 = *((u32 *)(0x80300000+4+it));

              v1 = (value1 & 0xFF000000) >> 24;
              v2 = (value2 & 0xFF000000) >> 24;
              v3 = (value3 & 0xFF000000) >> 24;
              v4 = (value4 & 0xFF000000) >> 24;
 
              result = ((v1 + v2 + v3 + v4) >> 2) << 24;

              v1 = (value1 & 0xFF0000) >> 16;
              v2 = (value2 & 0xFF0000) >> 16;
              v3 = (value3 & 0xFF0000) >> 16;
              v4 = (value4 & 0xFF0000) >> 16;
 
              result += ((v1 + v2 + v3 + v4) >> 2) << 16;

              v1 = (value1 & 0xFF00) >> 8;
              v2 = (value2 & 0xFF00) >> 8;
              v3 = (value3 & 0xFF00) >> 8;
              v4 = (value4 & 0xFF00) >> 8;
 
              result += ((v1 + v2 + v3 + v4) >> 2) << 8;

              v1 = (value1 & 0xFF);
              v2 = (value2 & 0xFF);
              v3 = (value3 & 0xFF);
              v4 = (value4 & 0xFF);
 
              result += ((v1 + v2 + v3 + v4) >> 2);

              *((u32 *)(0x80300000+it)) = result;
            }
	    /*	   
	    if ((loops % 10) == 0) { 
	    setcpos(30, 240 - 80);
	    printf("Song Position", mf->sngpos);
            printf("Pattern Position", mf->patpos);
            printf("Song Speed", mf->sngspd);
            printf("Song BPM", mf->bpm);
            printf("Counter", loops);
	    }
	    */
            if ((loops % 60) < 10) {
              setcpos(30, 240-50);
              puts("SONG NAME:");
              setcpos(30, 240-40);
              puts(mf->songname);
            }

	    //	    for (loops = 0; loops < 1000000; loops++) {}
	    //            loops++;
        }

        Player_Stop();      // stop playing.
        MikMod_FreeSong(mf);    // and free the module
        // puts("\n");

    puts("Exiting sound system");
    MikMod_Exit();       // deinitialize MikMod and sound card hardware

    puts("CZN Module Player terminated successfully.");

}










