#define DEBUG_DISPLAY
#ifdef DEBUG_DISPLAY
#include "types.h"
#endif

extern u8 font88[];

#ifdef DEBUG_DISPLAY
static u8 *scrbuf;
#endif
static u16 curx = 0, cury = 0;
#define SCRWIDTH 320
#define SCRHEIGHT 240

#define XOFFSET 30
#define YOFFSET 10

#ifdef DEBUG_DISPLAY
#define PRINTF(x, y) printf(x, y)
#else
#define PRINTF(x, y) null
#endif

void clrscr() {
#ifdef DEBUG_DISPLAY
  u32 it;

  curx = XOFFSET;
  cury = YOFFSET;

  for (it = 0; it < 320*240*4; it+=4) {
    *(scrbuf+0x20000000+it) = 0x00;
    *(scrbuf+0x20000000+it+1) = 0x00;
    *(scrbuf+0x20000000+it+2) = 0x00;
    *(scrbuf+0x20000000+it+3) = 0x00;
  }
#endif
}

void setscancol(unsigned long col) {
  u32 line =  *((u32 *)(0xA4400010));
  u32 it;

  line = (line & 0xFF)*320*4;

  for (it = 0; it < 320*4; it+= 4) {
    *((u32 *)(scrbuf + 0x20000000 + line + it)) = col;
  } 
}

u32 getscanline(void) {
  u32 line =  *((u32 *)(0xA4400010));
  line = (line & 0x000000FF);
  return line;
}

void setcpos(u16 x, u16 y) {
  curx = x;
  cury = y;
}

void putpixel(u16 x, u16 y, u8 r, u8 g, u8 b) {
#ifdef DEBUG_DISPLAY
  u32 value = 0;
  value = r;
  value = value << 8;
  value += g;
  value = value << 8;
  value += b;
  value = value << 8;
//  value++;
  *(u32 *)(scrbuf+x*4+y*320*4+0x20000000) = value;
#endif
}

void putchar(u16 x, u16 y, char c) {
#ifdef DEBUG_DISPLAY
  u16 itx, ity;
  char off = 255;

  if ((c >= 'A') && (c <= 'Z')) {
    off = c - 65;
  }

  if ((c >= 'a') && (c <= 'z')) {
    off = c - 97;
  }

  if ((c >= '0') && (c <= '9')) {
    off = c - 48 + 27;
  }

  if (c == ' ')
    off = 26;

  if (c == ':')
    off = 37;

  if (c == '+')
    off = 38;

  for (ity = 0; ity < 8; ity++) {
    for (itx = 0; itx < 8; itx++) {
      if (*(font88+(off)*128+itx*2+ity*16) == 0xFF)
        putpixel(x+itx, y+ity, 0xFF, 0xFF, 0xFF);
      else
        putpixel(x+itx, y+ity,  0,  0,  0);
    }
  }
#endif
}

void printstr(u16 x, u16 y, char *test) {
#ifdef DEBUG_DISPLAY
  u8 it = 0;
  while (*(test+it) != 0) {
    putchar(x+it*8, y, *(test+it));
    it++;
  }
#endif
}

void putc(char c) {
#ifdef DEBUG_DISPLAY
  putchar(curx, cury, c);
  curx += 8;
#endif
}

void puts(char *str) {
#ifdef DEBUG_DISPLAY
  printstr(curx, cury, str);
  curx = XOFFSET;
  cury += 8;
  if (cury >= SCRHEIGHT) {
    clrscr();
    cury = YOFFSET;
  }
#endif
}

void puthex(u32 c) {
#ifdef DEBUG_DISPLAY

  if (c < 10)
    putc(c + 48);
  if ((c >= 10) && (c < 16))
    putc(c + 65 - 10);
#endif
}

void print32(u32 valeur) {
#ifdef DEBUG_DISPLAY
  u32 temp, hex;

  temp = valeur;
  hex = temp & 0xF0000000;
  hex = hex >> 28;
  puthex(hex);
  hex = temp & 0x0F000000;
  hex = hex >> 24;
  puthex(hex);
  hex = temp & 0x00F00000;
  hex = hex >> 20;
  puthex(hex);
  hex = temp & 0x000F0000;
  hex = hex >> 16;
  puthex(hex);
  hex = temp & 0x0000F000;
  hex = hex >> 12;
  puthex(hex);
  hex = temp & 0x00000F00;
  hex = hex >> 8;
  puthex(hex);
  hex = temp & 0x000000F0;
  hex = hex >> 4;
  puthex(hex);
  hex = temp & 0x0000000F;
  puthex(hex);
#endif
}

void printf(char *text, u32 valeur) {
#ifdef DEBUG_DISPLAY
  int it;

  print32(valeur);
  putc(' ');
  puts(text);
#endif
}

void printfxy(u16 x, u16 y, char *text, u32 valeur) {
#ifdef DEBUG_DISPLAY
  int it;

  setcpos(x, y);
  print32(valeur);
  putc(' ');
  puts(text);
#endif
}


void initscrbuf(u8 *screenaddr) {
#ifdef DEBUG_DISPLAY

  scrbuf = screenaddr;

  clrscr();
  curx = XOFFSET; cury = YOFFSET;

#endif
}
