/* Memory management */
#include "mikmod.h"

#define MIKMODHEAPSIZE 1500000
static char MikModHeap[MIKMODHEAPSIZE];
char *MikModPtr;
static unsigned long randseed = 1;

#define u32 unsigned long

char *malloc(u32 size) {
  char *result;
  u32 temp;
  u32 newsize = size + 32;
  if (MikModPtr + newsize <= MikModHeap+MIKMODHEAPSIZE) {
    temp = (u32)MikModPtr+16;
    result = (char*) ((temp) & 0xFFFFFFF0);
    //    result += 0x20000000;
    MikModPtr += newsize;
    memset(result, 0, size);
    //    printf("RESULT OF MALLOC", result);
    //    printf("SIZE", size);
  }
  else {
#ifdef DEBUG_DISPLAY
    printf("OUT OF MEMORY", 0xFFFFFFFF);
    while (1) {};
#endif
    result = NULL;
//  BREAK;
  }
  return result;
}

char *calloc(u32 nitems, u32 size) {
  return malloc(nitems*size);
}

char *memset(char *s, char c, size_t n) {
  int it;

  for (it = 0; it < n; it++)
    *(char *)(s+it) = c;

  return s;
}

char *memcpy(char *dest, char *src, size_t n) {
  size_t it;

  for (it = 0; it < n; it++)
    *(char *)(dest+it) = *(char *)(src+it);

  return dest;
}

int memcmp(const void *s1, const void *s2, size_t n) {
  size_t it;
  int result; // -1 = less, =0 equal, =1 greater

  result = 0;
  for (it = 0; it < n; it++) {
    if (*(char *)(s1+it) == *(char *)(s2+it)) {
    }
    if (*(char *)(s1+it) < *(char *)(s2+it)) {
      result = -1;
      break;
    }
    if (*(char *)(s1+it) > *(char *)(s2+it)) {
      result = 1;
      break;
    }
  }

  return result;
}

size_t strlen(const char *s) {
  size_t result = 0;

  while (*(char *)(s+result) != 0) 
    result++;

  return result;
}

char *strdup(const char *s) {
  char *newstr = malloc(strlen(s));

  memcpy(newstr, s, strlen(s));

  return newstr;
}

void srand(unsigned int seed) {
  randseed = seed;
}

/*
 * Pseudo-random number generator for randomizing the profiling clock,
 * and whatever else we might use it for.  The result is uniform on
 * [0, 2^31 - 1].
 */
#define RAND_MAX 2147483647
int rand(void)
{
        register long x, hi, lo, t;

        /*
         * Compute x[n + 1] = (7^5 * x[n]) mod (2^31 - 1).
         * From "Random number generators: good ones are hard to find",
         * Park and Miller, Communications of the ACM, vol. 31, no. 10,
         * October 1988, p. 1195.
         */
        x = randseed;
        hi = x / 127773;
        lo = x % 127773;
        t = 16807 * lo - 2836 * hi;
        if (t <= 0)
                t += 0x7fffffff;
        randseed = (t % RAND_MAX);
        return (t);
} 

void *realloc(void *ptr, size_t size) {
  char *result;

  if (ptr == NULL) {
    result = malloc(size);
  } else {
    if (size == 0) {
      free(ptr);
      result = ptr; 
    } else {
      result = malloc(size);
      memcpy(result, ptr, size);
    }
  }

  return result;
}

int free(char *memaddr) {
  return 0;
}

void initmem() {
  MikModPtr = MikModHeap;
}
